@@ -1,7 +1,7 @@
 package SQL::Translator::Parser::MySQL;
 
 # -------------------------------------------------------------------
-# $Id: MySQL.pm,v 1.55 2006/10/10 19:04:54 duality72 Exp $
+# $Id: MySQL.pm,v 1.58 2007-03-19 17:15:24 duality72 Exp $
 # -------------------------------------------------------------------
 # Copyright (C) 2002-4 SQLFairy Authors
 #
@@ -101,9 +101,12 @@ Here's the word from the MySQL site
   
   table_options:
           TYPE = {BDB | HEAP | ISAM | InnoDB | MERGE | MRG_MYISAM | MYISAM }
+  or      ENGINE = {BDB | HEAP | ISAM | InnoDB | MERGE | MRG_MYISAM | MYISAM }
   or      AUTO_INCREMENT = #
   or      AVG_ROW_LENGTH = #
+  or      [ DEFAULT ] CHARACTER SET charset_name
   or      CHECKSUM = {0 | 1}
+  or      COLLATE collation_name
   or      COMMENT = "string"
   or      MAX_ROWS = #
   or      MIN_ROWS = #
@@ -117,6 +120,7 @@ Here's the word from the MySQL site
   or      DATA DIRECTORY="absolute path to directory"
   or      INDEX DIRECTORY="absolute path to directory"
 
+
 A subset of the ALTER TABLE syntax that allows addition of foreign keys:
 
   ALTER [IGNORE] TABLE tbl_name alter_specification [, alter_specification] ...
@@ -134,14 +138,19 @@ A subset of INSERT that we ignore:
 
 use strict;
 use vars qw[ $DEBUG $VERSION $GRAMMAR @EXPORT_OK ];
-$VERSION = sprintf "%d.%02d", q$Revision: 1.55 $ =~ /(\d+)\.(\d+)/;
+$VERSION = sprintf "%d.%02d", q$Revision: 1.58 $ =~ /(\d+)\.(\d+)/;
 $DEBUG   = 0 unless defined $DEBUG;
 
 use Data::Dumper;
 use Parse::RecDescent;
 use Exporter;
+use Storable qw(dclone);
+use DBI qw(:sql_types);
 use base qw(Exporter);
 
+our %type_mapping = (
+);
+
 @EXPORT_OK = qw(parse);
 
 # Enable warnings within the Parse::RecDescent module.
@@ -149,10 +158,12 @@ $::RD_ERRORS = 1; # Make sure the parser dies when it encounters an error
 $::RD_WARN   = 1; # Enable warnings. This will warn on unused rules &c.
 $::RD_HINT   = 1; # Give out hints to help fix problems.
 
+use constant DEFAULT_PARSER_VERSION => 30000;
+
 $GRAMMAR = << 'END_OF_GRAMMAR';
 
 { 
-    my ( $database_name, %tables, $table_order, @table_comments );
+    my ( $database_name, %tables, $table_order, @table_comments, %views, $view_order, %procedures, $proc_order );
     my $delimiter = ';';
 }
 
@@ -163,7 +174,7 @@ $GRAMMAR = << 'END_OF_GRAMMAR';
 # failed. -ky
 #
 startrule : statement(s) eofile { 
-    { tables => \%tables, database_name => $database_name } 
+    { tables => \%tables, database_name => $database_name, views => \%views, procedures =>\%procedures } 
 }
 
 eofile : /^\Z/
@@ -209,7 +220,7 @@ statement_body : (string | nonstring)(s?)
 insert : /insert/i  statement_body "$delimiter"
 
 delimiter : /delimiter/i /[\S]+/
-	{ $delimiter = $item[2] }
+    { $delimiter = $item[2] }
 
 empty_statement : "$delimiter"
 
@@ -295,6 +306,48 @@ create : CREATE UNIQUE(?) /(index|key)/i index_name /on/i table_name '(' field_n
         ;
     }
 
+create : CREATE /trigger/i NAME not_delimiter "$delimiter"
+    {
+        @table_comments = ();
+    }
+
+create : CREATE PROCEDURE NAME not_delimiter "$delimiter"
+    {
+        @table_comments = ();
+        my $func_name = $item[3];
+        my $owner = '';
+        my $sql = "$item[1] $item[2] $item[3] $item[4]";
+        
+        $procedures{ $func_name }{'order'}  = ++$proc_order;
+        $procedures{ $func_name }{'name'}   = $func_name;
+        $procedures{ $func_name }{'owner'}  = $owner;
+        $procedures{ $func_name }{'sql'}    = $sql;
+    }
+
+PROCEDURE : /procedure/i
+    | /function/i
+
+create : CREATE algorithm /view/i NAME not_delimiter "$delimiter"
+    {
+        @table_comments = ();
+        my $view_name = $item[4];
+        my $sql = "$item[1] $item[2] $item[3] $item[4] $item[5]";
+        
+        # Hack to strip database from function calls in SQL
+        $sql =~ s#`\w+`\.(`\w+`\()##g;
+        
+        $views{ $view_name }{'order'}  = ++$view_order;
+        $views{ $view_name }{'name'}   = $view_name;
+        $views{ $view_name }{'sql'}    = $sql;
+    }
+
+algorithm : /algorithm/i /=/ WORD
+    {
+        $return = "$item[1]=$item[3]";
+    }
+
+not_delimiter : /.*?(?=$delimiter)/is
+
 create_definition : constraint 
     | index
     | field
@@ -309,9 +362,10 @@ comment : /^\s*(?:#|-{2}).*\n/
         $return     = $comment;
     }
 
-comment : /\/\*/ /[^\*]+/ /\*\//
+comment : /\/\*/ /.*?\*\//s
     {
         my $comment = $item[2];
+        $comment = substr($comment, 0, -2);
         $comment    =~ s/^\s*|\s*$//g;
         $return = $comment;
     }
@@ -490,42 +544,6 @@ data_type    : WORD parens_value_list(s?) type_qualifier(s?)
             $list = [];
         }
 
-        unless ( @{ $size || [] } ) {
-            if ( lc $type eq 'tinyint' ) {
-                $size = 4;
-            }
-            elsif ( lc $type eq 'smallint' ) {
-                $size = 6;
-            }
-            elsif ( lc $type eq 'mediumint' ) {
-                $size = 9;
-            }
-            elsif ( $type =~ /^int(eger)?$/i ) {
-                $type = 'int';
-                $size = 11;
-            }
-            elsif ( lc $type eq 'bigint' ) {
-                $size = 20;
-            }
-            elsif ( 
-                lc $type =~ /(float|double|decimal|numeric|real|fixed|dec)/ 
-            ) {
-                $size = [8,2];
-            }
-        }
-
-        if ( $type =~ /^tiny(text|blob)$/i ) {
-            $size = 255;
-        }
-        elsif ( $type =~ /^(blob|text)$/i ) {
-            $size = 65_535;
-        }
-        elsif ( $type =~ /^medium(blob|text)$/i ) {
-            $size = 16_777_215;
-        }
-        elsif ( $type =~ /^long(blob|text)$/i ) {
-            $size = 4_294_967_295;
-        }
 
         $return        = { 
             type       => $type,
@@ -663,10 +681,14 @@ table_option : /comment/i /=/ /'.*?'/
         $comment    =~ s/'$//;
         $return     = { comment => $comment };
     }
-    | /(default )?(charset|character set)/i /\s*=\s*/ WORD
+    | /(default )?(charset|character set)/i /\s*=?\s*/ WORD
     { 
         $return = { 'CHARACTER SET' => $item[3] };
     }
+    | /collate/i WORD
+    {
+        $return = { 'COLLATE' => $item[2] }
+    }
     | WORD /\s*=\s*/ WORD
     { 
         $return = { $item[1] => $item[3] };
@@ -690,7 +712,9 @@ DIGITS : /\d+/
 
 COMMA : ','
 
-NAME    : "`" /\w+/ "`"
+BACKTICK : '`'
+
+NAME    : BACKTICK /\w+/ BACKTICK
     { $item[2] }
     | /\w+/
     { $item[1] }
@@ -708,9 +732,9 @@ VALUE   : /[-+]?\.?\d+(?:[eE]\d+)?/
     { 'NULL' }
 
 CURRENT_TIMESTAMP : /current_timestamp(\(\))?/i
-	| /now\(\)/i
-	{ 'CURRENT_TIMESTAMP' }
-	
+    | /now\(\)/i
+    { 'CURRENT_TIMESTAMP' }
+    
 END_OF_GRAMMAR
 
 # -------------------------------------------------------------------
@@ -725,6 +749,10 @@ sub parse {
         return $translator->error("Error instantiating Parse::RecDescent ".
             "instance: Bad grammer");
     }
+    
+    # Preprocess for MySQL-specific and not-before-version comments from mysqldump
+    my $parser_version = $translator->parser_args->{mysql_parser_version} || DEFAULT_PARSER_VERSION;
+    while ( $data =~ s#/\*!(\d{5})?(.*?)\*/#($1 && $1 > $parser_version ? '' : $2)#es ) {}
 
     my $result = $parser->startrule($data);
     return $translator->error( "Parse failed." ) unless defined $result;
@@ -768,7 +796,7 @@ sub parse {
             $table->primary_key( $field->name ) if $fdata->{'is_primary_key'};
 
             for my $qual ( qw[ binary unsigned zerofill list collate ],
-            		'character set', 'on update' ) {
+                    'character set', 'on update' ) {
                 if ( my $val = $fdata->{ $qual } || $fdata->{ uc $qual } ) {
                     next if ref $val eq 'ARRAY' && !@$val;
                     $field->extra( $qual, $val );
@@ -791,20 +819,12 @@ sub parse {
                 ) or die $table->error;
             }
 
-            if ( $field->data_type =~ /(set|enum)/i && !$field->size ) {
-                my %extra = $field->extra;
-                my $longest = 0;
-                for my $len ( map { length } @{ $extra{'list'} || [] } ) {
-                    $longest = $len if $len > $longest;
-                }
-                $field->size( $longest ) if $longest;
-            }
-
             for my $cdata ( @{ $fdata->{'constraints'} } ) {
                 next unless $cdata->{'type'} eq 'foreign_key';
                 $cdata->{'fields'} ||= [ $field->name ];
                 push @{ $tdata->{'constraints'} }, $cdata;
             }
+
         }
 
         for my $idata ( @{ $tdata->{'indices'} || [] } ) {
@@ -831,11 +851,114 @@ sub parse {
                 on_update        => $cdata->{'on_update'} || $cdata->{'on_update_do'},
             ) or die $table->error;
         }
+
+        # After the constrains and PK/idxs have been created, we normalize fields
+        normalize_field($_) for $table->get_fields;
+    }
+    
+    my @procedures = sort { 
+        $result->{procedures}->{ $a }->{'order'} <=> $result->{procedures}->{ $b }->{'order'}
+    } keys %{ $result->{procedures} };
+    foreach my $proc_name (@procedures) {
+        $schema->add_procedure(
+            name  => $proc_name,
+            owner => $result->{procedures}->{$proc_name}->{owner},
+            sql   => $result->{procedures}->{$proc_name}->{sql},
+        );
+    }
+
+    my @views = sort { 
+        $result->{views}->{ $a }->{'order'} <=> $result->{views}->{ $b }->{'order'}
+    } keys %{ $result->{views} };
+    foreach my $view_name (keys %{ $result->{views} }) {
+        $schema->add_view(
+            name => $view_name,
+            sql  => $result->{views}->{$view_name}->{sql},
+        );
     }
 
     return 1;
 }
 
+# Takes a field, and returns 
+sub normalize_field {
+    my ($field) = @_;
+    my ($size, $type, $list, $changed) = @_;
+  
+    $size = $field->size;
+    $type = $field->data_type;
+    $list = $field->extra->{list} || [];
+
+    if ( !ref $size && $size eq 0 ) {
+        if ( lc $type eq 'tinyint' ) {
+            $changed = $size != 4;
+            $size = 4;
+        }
+        elsif ( lc $type eq 'smallint' ) {
+            $changed = $size != 6;
+            $size = 6;
+        }
+        elsif ( lc $type eq 'mediumint' ) {
+            $changed = $size != 9;
+            $size = 9;
+        }
+        elsif ( $type =~ /^int(eger)?$/i ) {
+            $changed = $size != 11 || $type ne 'int';
+            $type = 'int';
+            $size = 11;
+        }
+        elsif ( lc $type eq 'bigint' ) {
+            $changed = $size != 20;
+            $size = 20;
+        }
+        elsif ( lc $type =~ /(float|double|decimal|numeric|real|fixed|dec)/ ) {
+            my $old_size = (ref $size || '') eq 'ARRAY' ? $size : [];
+            $changed = @$old_size != 2 || $old_size->[0] != 8 || $old_size->[1] != 2;
+            $size = [8,2];
+        }
+    }
+
+    if ( $type =~ /^tiny(text|blob)$/i ) {
+        $changed = $size != 255;
+        $size = 255;
+    }
+    elsif ( $type =~ /^(blob|text)$/i ) {
+        $changed = $size != 65_535;
+        $size = 65_535;
+    }
+    elsif ( $type =~ /^medium(blob|text)$/i ) {
+        $changed = $size != 16_777_215;
+        $size = 16_777_215;
+    }
+    elsif ( $type =~ /^long(blob|text)$/i ) {
+        $changed = $size != 4_294_967_295;
+        $size = 4_294_967_295;
+    }
+    if ( $field->data_type =~ /(set|enum)/i && !$field->size ) {
+        my %extra = $field->extra;
+        my $longest = 0;
+        for my $len ( map { length } @{ $extra{'list'} || [] } ) {
+            $longest = $len if $len > $longest;
+        }
+        $changed = 1;
+        $size = $longest if $longest;
+    }
+
+
+    if ($changed) {
+      # We only want to clone the field, not *everything*
+      { local $field->{table} = undef;
+        $field->parsed_field(dclone($field));
+        $field->parsed_field->{table} = $field->table;
+      }
+      $field->size($size);
+      $field->data_type($type);
+      $field->sql_data_type( $type_mapping{lc $type} ) if exists $type_mapping{lc $type};
+      $field->extra->{list} = $list if @$list;
+    }
+}
+
+
 1;
 
 # -------------------------------------------------------------------
